@@ -33,41 +33,46 @@
 #'
 #' @param x a greta array
 #' @param i,j indices specifying elements to extract or replace
-#' @param n a single integer, as in \code{utils::head()} and
-#'   \code{utils::tail()}
+#' @param n a single integer, as in `utils::head()` and
+#'   `utils::tail()`
 #' @param nrow,ncol optional dimensions for the resulting greta array when x is
 #'   not a matrix.
-#' @param value for \code{`[<-`} a greta array to replace elements, for
-#'   \code{`dim<-`} either NULL or a numeric vector of dimensions
+#' @param value for ``[<-`` a greta array to replace elements, for
+#'   ``dim<-`` either NULL or a numeric vector of dimensions
 #' @param ... either further indices specifying elements to extract or replace
-#'   (\code{[}), or multiple greta arrays to combine (\code{cbind()},
-#'   \code{rbind()} & \code{c()}), or additional arguments (\code{rep()},
-#'   \code{head()}, \code{tail()})
+#'   (`[`), or multiple greta arrays to combine (`cbind()`,
+#'   `rbind()` & `c()`), or additional arguments (`rep()`,
+#'   `head()`, `tail()`)
 #' @param drop,recursive generic arguments that are ignored for greta arrays
 #'
-#' @details \code{diag()} can be used to extract or replace the diagonal part of
+#' @details `diag()` can be used to extract or replace the diagonal part of
 #'   a square and two-dimensional greta array, but it cannot be used to create a
 #'   matrix-like greta array from a scalar or vector-like greta array. A static
-#'   diagonal matrix can always be created with e.g. \code{diag(3)}, and then
+#'   diagonal matrix can always be created with e.g. `diag(3)`, and then
 #'   converted into a greta array.
 #'
+#'   Also note that since R 4.0.0, `head` and `tail` methods for arrays changed
+#'   to print a vector rather than maintain the array structure. The `greta`
+#'   package supports both methods, and will do so based on which version of R
+#'    you are using.
+#'
 #' @examples
 #' \dontrun{
 #'
-#'  x <- as_data(matrix(1:12, 3, 4))
+#' x <- as_data(matrix(1:12, 3, 4))
 #'
-#'  # extract and replace
-#'  x[1:3, ]
-#'  x[, 2:4] <- 1:9
-#'  e <- diag(x)
-#'  diag(x) <- e + 1
+#' # extract and replace
+#' x[1:3, ]
+#' x[, 2:4] <- 1:9
+#' e <- diag(x)
+#' diag(x) <- e + 1
 #'
-#'  # combine
-#'  cbind(x[, 2], x[, 1])
-#'  rbind(x[1, ], x[3, ])
-#'  abind(x[1, ], x[3, ], along = 1)
-#'  c(x[, 1], x)
-#'  rep(x[, 2], times = 3)
+#' # combine
+#' cbind(x[, 2], x[, 1])
+#' rbind(x[1, ], x[3, ])
+#' abind(x[1, ], x[3, ], along = 1)
+#' c(x[, 1], x)
+#' rep(x[, 2], times = 3)
 #' }
 NULL
 
@@ -96,14 +101,18 @@ NULL
   rm("._dummy_in", envir = pf)
 
   if (any(is.na(dummy_out))) {
-    stop("subscript out of bounds",
-         call. = FALSE)
+    msg <- cli::format_error(
+      "subscript out of bounds"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
   }
 
   node <- get_node(x)
   # if this is a data node, also subset the values and pass on
   if (inherits(node, "data_node")) {
-
     values_in <- node$value()
     call_list <- as.list(call)[-1]
     call_list[[1]] <- as.name("._values_in")
@@ -118,11 +127,8 @@ NULL
 
     # make sure it's an array
     values <- as.array(values_out)
-
   } else {
-
     values <- NULL
-
   }
 
   # coerce result to an array
@@ -133,37 +139,46 @@ NULL
   dims_out <- dim(dummy_out)
 
   # make sure it's a column vector
-  if (length(dims_out) == 1)
+  if (length(dims_out) == 1) {
     dims_out <- c(dims_out, 1)
+  }
 
   # give values these dimensions
-  if (is.array(values))
+  if (is.array(values)) {
     values <- array(values, dim = dims_out)
+  }
 
   # get the index in flat python format, as a tensor
   index <- flatten_rowwise(dummy_out)
 
   # create operation node, passing call and dims as additional arguments
   op("extract",
-     x,
-     dim = dims_out,
-     operation_args = list(nelem = nelem,
-                           index = index,
-                           dims_out = dims_out),
-     tf_operation = "tf_extract",
-     value = values)
-
+    x,
+    dim = dims_out,
+    operation_args = list(
+      nelem = nelem,
+      index = index,
+      dims_out = dims_out
+    ),
+    tf_operation = "tf_extract",
+    value = values
+  )
 }
 
 # replace syntax for greta array objects
 #' @export
-`[<-.greta_array` <- function(x, ..., value) {
+`[<-.greta_array` <- function(x, ..., value) { # nolint
 
   node <- get_node(x)
 
   if (inherits(node, "variable_node")) {
-    stop("cannot replace values in a variable greta array",
-         call. = FALSE)
+    msg <- cli::format_error(
+      "cannot replace values in a variable {.cls greta_array}"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
   }
 
   # rename value since it is confusing when passed to the op
@@ -190,23 +205,28 @@ NULL
   rm("._dummy_in", envir = pf)
 
   if (any(is.na(dummy_out))) {
-    stop("subscript out of bounds",
-         call. = FALSE)
+    msg <- cli::format_error(
+      "subscript out of bounds"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
   }
 
   index <- as.vector(dummy_out)
 
   if (length(index) != length(replacement)) {
-
     if (length(index) %% length(replacement) != 0) {
-
-      stop("number of items to replace is not a multiple of ",
-           "replacement length")
-
+      msg <- cli::format_error(
+        "number of items to replace is not a multiple of replacement length"
+      )
+      stop(
+        msg,
+        call. = FALSE
+      )
     } else {
-
       replacement <- rep(replacement, length.out = length(index))
-
     }
   }
 
@@ -219,80 +239,107 @@ NULL
   new_value[r_index] <- replacement_value
 
   # if either parent has an unknowns array as a value, coerce this to unknowns
-  if (inherits(x_value, "unknowns") | inherits(replacement_value, "unknowns"))
+  if (inherits(x_value, "unknowns") | inherits(replacement_value, "unknowns")) {
     new_value <- as.unknowns(new_value)
+  }
 
   # create operation node, passing call and dims as additional arguments
   op("replace",
-     x,
-     replacement,
-     dim = dims,
-     operation_args = list(index = index,
-                           dims = dims),
-     value = new_value,
-     tf_operation = "tf_replace")
-
+    x,
+    replacement,
+    dim = dims,
+    operation_args = list(
+      index = index,
+      dims = dims
+    ),
+    value = new_value,
+    tf_operation = "tf_replace"
+  )
 }
 
 #' @export
 cbind.greta_array <- function(...) {
-
   dots <- list(...)
   dots <- lapply(dots, as.greta_array)
 
   dims <- lapply(dots, dim)
   ndims <- vapply(dims, length, FUN.VALUE = 1)
-  if (!all(ndims == 2))
-    stop("all greta arrays must be two-dimensional")
+  if (!all(ndims == 2)) {
+    msg <- cli::format_error(
+      "all {.cls greta_array}s must be two-dimensional"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
+  }
 
   # dimensions
   rows <- vapply(dims, `[`, 1, FUN.VALUE = 1)
   cols <- vapply(dims, `[`, 2, FUN.VALUE = 1)
 
   # check all the same
-  if (!all(rows == rows[1]))
-    stop("all greta arrays must be have the same number of rows",
-         call. = FALSE)
+  if (!all(rows == rows[1])) {
+    msg <- cli::format_error(
+      "all {.cls greta_array}s must be have the same number of rows"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
+  }
 
   # output dimensions
   dims <- c(rows[1], sum(cols))
 
   op("cbind", ...,
-     dim = dims,
-     tf_operation = "tf_cbind")
-
+    dim = dims,
+    tf_operation = "tf_cbind"
+  )
 }
 
 #' @export
 rbind.greta_array <- function(...) {
-
   dots <- list(...)
   dots <- lapply(dots, as.greta_array)
 
   dims <- lapply(dots, dim)
   ndims <- vapply(dims, length, FUN.VALUE = 1)
-  if (!all(ndims == 2))
-    stop("all greta arrays must be two-dimensional")
+  if (!all(ndims == 2)) {
+    msg <- cli::format_error(
+      "all {.cls greta_array}s must be two-dimensional"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
+  }
 
   # dimensions
   rows <- vapply(dims, `[`, 1, FUN.VALUE = 1)
   cols <- vapply(dims, `[`, 2, FUN.VALUE = 1)
 
   # check all the same
-  if (!all(cols == cols[1]))
-    stop("all greta arrays must be have the same number of columns",
-         call. = FALSE)
+  if (!all(cols == cols[1])) {
+    msg <- cli::format_error(
+      "all {.cls greta_array}s must be have the same number of columns"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
+  }
 
   # output dimensions
   dims <- c(sum(rows), cols[1])
 
   op("rbind", ...,
-     dim = dims,
-     tf_operation = "tf_rbind")
-
+    dim = dims,
+    tf_operation = "tf_rbind"
+  )
 }
 
-# Begin Exclude Linting
+# nolint start
 #' @rdname overloaded
 #' @export
 abind <- function(...,
@@ -302,36 +349,40 @@ abind <- function(...,
                   hier.names = FALSE, use.dnns = FALSE) {
   UseMethod("abind")
 }
-# End Exclude Linting
+# nolint end
 
-# clear CRAN checks spotting floating global variables
-#' @importFrom utils globalVariables
-utils::globalVariables("N", "greta")
-
-# Begin Exclude Linting
+# nolint start
 #' @export
 abind.default <- function(...,
                           along = N, rev.along = NULL, new.names = NULL,
                           force.array = TRUE, make.names = use.anon.names,
                           use.anon.names = FALSE, use.first.dimnames = FALSE,
                           hier.names = FALSE, use.dnns = FALSE) {
+  # nolint end
 
-  # End Exclude Linting
   # error nicely if they don't have abind installed
   abind_installed <- requireNamespace("abind", quietly = TRUE)
   if (!abind_installed) {
-    stop("abind is being called on R arrays (not greta arrays), ",
-         "but the abind package is not installed",
-         call. = FALSE)
+    msg <- cli::format_error(
+      c(
+        "{.fun abind} is being called on R arrays (not {.cls greta_array}s), \\
+        but the {.pkg abind} package is not installed",
+        "install {.pkg abind} with:",
+        "{.code install.packages('abind')}"
+      )
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
   }
 
   call <- sys.call()
   call_list <- as.list(call)[-1]
   do.call(abind::abind, call_list)
-
 }
 
-# Begin Exclude Linting
+# nolint start
 #' @export
 abind.greta_array <- function(...,
                               along = N, rev.along = NULL, new.names = NULL,
@@ -339,8 +390,10 @@ abind.greta_array <- function(...,
                               use.anon.names = FALSE,
                               use.first.dimnames = FALSE, hier.names = FALSE,
                               use.dnns = FALSE) {
+  # nolint end
 
   # warn if any of the arguments have been changed
+  # nolint start
   user_set_args <- !is.null(rev.along) |
     !is.null(new.names) |
     !isTRUE(force.array) |
@@ -349,64 +402,87 @@ abind.greta_array <- function(...,
     !identical(use.first.dimnames, FALSE) |
     !identical(hier.names, FALSE) |
     !identical(use.dnns, FALSE)
-
-  # End Exclude Linting
+  # nolint end
 
   if (user_set_args) {
-    warning("only the argument 'along' is supported when using abind ",
-            "with greta arrays, any other arguments will be ignored")
+    msg <- cli::format_warning(
+      "only the argument {.arg along} is supported when using {.fun abind} \\
+      with {.cls greta_array}s, any other arguments will be ignored"
+    )
+    warning(
+      msg,
+      call. = FALSE
+    )
   }
 
-  arg.list <- list(...)
+  arg_list <- list(...)
 
   # drop any NULLs
-  to_discard <- vapply(arg.list, is.null, FUN.VALUE = FALSE)
+  to_discard <- vapply(arg_list, is.null, FUN.VALUE = FALSE)
   if (any(to_discard)) {
-    arg.list <- arg.list[!to_discard]
+    arg_list <- arg_list[!to_discard]
   }
 
   # get N first, in case they used the default value for along
-  dims <- lapply(arg.list, dim)
-  N <- max(vapply(dims, length, FUN.VALUE = 1L))
+  dims <- lapply(arg_list, dim)
+  n <- max(vapply(dims, length, FUN.VALUE = 1L))
+
+  # needed to keep the same formals as abind
+  N <- n # nolint
   along <- as.integer(force(along))
 
   # rationalise along, and pad N if we're prepending/appending a dimension
-  if (along < 1 || along > N || (along > floor(along) &&
-                                 along < ceiling(along))) {
-    N <- N + 1
-    along <- max(1, min(N + 1, ceiling(along)))
+  if (along < 1 || along > n || (along > floor(along) &&
+    along < ceiling(along))) {
+    n <- n + 1
+    along <- max(1, min(n + 1, ceiling(along)))
   }
 
-  if (!along %in% 0:N) {
-    stop("along must be between 0 and ", N, call. = FALSE)
+  if (!(along %in% 0:n)) {
+    msg <- cli::format_error(
+      c(
+        "{.arg along} must be between 0 and {n}",
+        "Instead {.arg along} was {.val {along}}"
+      )
+    )
+    stop(
+      msg,
+      call. = FALSE
+      )
   }
 
   pre <- seq(from = 1, len = along - 1)
-  post <- seq(to = N - 1, len = N - along)
+  post <- seq(to = n - 1, len = n - along)
 
-
-  # rationalise the dimensions
-  arg.list <- arg.list
-
-  # loop though all elements of arg.list padding if necessary:
-  arg.list <- lapply(arg.list,
-                     function(x) {
-                       dim <- dim(x)
-                       if (length(dim) == N - 1)
-                         dim(x) <- c(dim[pre], 1, dim[post])
-                       x
-                     })
+  # loop though all elements of arg_list padding if necessary:
+  arg_list <- lapply(
+    arg_list,
+    function(x) {
+      dim <- dim(x)
+      if (length(dim) == n - 1) {
+        dim(x) <- c(dim[pre], 1, dim[post])
+      }
+      x
+    }
+  )
 
   # check the non-binding dimensions match
-  dims <- lapply(arg.list, dim)
-  dim_out <- rep(NA, N)
-  for (dim in seq_len(N)[-along]) {
+  dims <- lapply(arg_list, dim)
+  dim_out <- rep(NA, n)
+  for (dim in seq_len(n)[-along]) {
     this_dim <- vapply(dims, `[`, dim, FUN.VALUE = 1L)
     if (!all(this_dim == this_dim[1])) {
-      stop("all greta arrays must have the same dimensions ",
-           "except on the 'along' dimension, but dimension ", dim,
-           " had varying sizes: ", paste(this_dim, collapse = ", "),
-           call. = FALSE)
+      msg <- cli::format_error(
+        c(
+          "all {.cls greta_array}s must have the same dimensions except on \\
+          the {.arg along} dimension",
+          "However, dimension {dim} had varying sizes: {this_dim}"
+        )
+      )
+      stop(
+        msg,
+        call. = FALSE
+      )
     } else {
       dim_out[dim] <- this_dim[1]
     }
@@ -415,18 +491,20 @@ abind.greta_array <- function(...,
   bind_dims <- vapply(dims, `[`, along, FUN.VALUE = 1L)
   dim_out[along] <- sum(bind_dims)
 
-  do.call(op,
-          c(operation = "abind",
-            arg.list,
-            dim = list(dim_out),
-            operation_args = list(list(axis = as.integer(along))),
-            tf_operation = "tf_abind"))
-
+  do.call(
+    op,
+    c(
+      operation = "abind",
+      arg_list,
+      dim = list(dim_out),
+      operation_args = list(list(axis = as.integer(along))),
+      tf_operation = "tf_abind"
+    )
+  )
 }
 
 #' @export
 c.greta_array <- function(...) {
-
   args <- list(...)
 
   # drop NULLs from the list
@@ -438,11 +516,13 @@ c.greta_array <- function(...) {
 
   # return a list if they aren't all greta arrays
   is_greta_array <- vapply(args,
-                           inherits, "greta_array",
-                           FUN.VALUE = FALSE)
+    inherits, "greta_array",
+    FUN.VALUE = FALSE
+  )
 
-  if (!all(is_greta_array))
+  if (!all(is_greta_array)) {
     return(args)
+  }
 
   # loop through arrays, flattening them R-style
   arrays <- lapply(args, flatten)
@@ -452,12 +532,15 @@ c.greta_array <- function(...) {
   dim_out <- c(sum(length_vec), 1L)
 
   # create the op, expanding 'arrays' out to match op()'s dots input
-  do.call(op,
-          c(operation = "rbind",
-            arrays,
-            dim = list(dim_out),
-            tf_operation = "tf_rbind"))
-
+  do.call(
+    op,
+    c(
+      operation = "rbind",
+      arrays,
+      dim = list(dim_out),
+      tf_operation = "tf_rbind"
+    )
+  )
 }
 
 #' @export
@@ -468,160 +551,237 @@ rep.greta_array <- function(x, ...) {
 
   # apply (implicitly coercing to a column vector)
   x[idx]
-
 }
 
 # get dimensions
 #' @export
-dim.greta_array <- function(x)
+dim.greta_array <- function(x) {
   as.integer(get_node(x)$dim)
+}
 
 #' @export
-length.greta_array <- function(x)
+length.greta_array <- function(x) {
   prod(dim(x))
+}
 
 # reshape greta arrays
 #' @export
-`dim<-.greta_array` <- function(x, value) {
+`dim<-.greta_array` <- function(x, value) { # nolint
 
   dims <- value
 
-  if (is.null(dims))
+  if (is.null(dims)) {
     dims <- length(x)
+  }
 
-  if (length(dims) == 0L)
-    stop("length-0 dimension vector is invalid",
-         call. = FALSE)
+  if (length(dims) == 0L) {
+    msg <- cli::format_error(
+      "length-0 dimension vector is invalid"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
+  }
 
-  if (length(dims) == 1L)
+  if (length(dims) == 1L) {
     dims <- c(dims, 1L)
+  }
 
-  if (any(is.na(dims)))
-    stop("the dims contain missing values",
-         call. = FALSE)
+  if (any(is.na(dims))) {
+    msg <- cli::format_error(
+      "the dims contain missing values"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
+  }
 
   dims <- as.integer(dims)
 
-  if (any(dims < 0L))
-    stop("the dims contain negative values",
-         call. = FALSE)
+  if (any(dims < 0L)) {
+    msg <- cli::format_error(
+      c(
+        "the dims contain negative values:",
+        "{.code dim(x)} returns {dim(x)}"
+      )
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
+  }
 
   prod_dims <- prod(dims)
   len <- length(x)
 
-  if (prod_dims != len) {
-    msg <- sprintf("dims [product %i] do not match the length of object [%i]",
-                   prod_dims, len)
-    stop(msg, call. = FALSE)
+  # if x isn't a scalar and the numbers of elements don't match, error
+  if (len != 1 && prod_dims != len) {
+    msg <- cli::format_error(
+      c(
+        "dims [product {prod_dims}] do not match the length of object [{len}]"
+      )
+    )
+    stop(
+      msg,
+      call. = FALSE
+      )
   }
 
+  # change the values similarly
   new_value <- get_node(x)$value()
-  dim(new_value) <- dims
+  new_value <- array(new_value, dim = dims)
 
-  op("set_dim",
-     x,
-     operation_args = list(dims = dims),
-     tf_operation = "tf_set_dim",
-     dim = dims,
-     value = new_value)
+  if (!identical(dim(x), dims) && len == 1) {
 
+    # if the dims don't match, but x is a scalar, expand it to the required
+    # dimension
+    op("expand_dim",
+      x,
+      operation_args = list(dims = dims),
+      tf_operation = "tf_expand_dim",
+      dim = dims,
+      value = new_value
+    )
+  } else {
+    # otherwise, if the dimensions don't match, but the number of elements do,
+    # just change the dimensions
+    op("set_dim",
+      x,
+      operation_args = list(dims = dims),
+      tf_operation = "tf_set_dim",
+      dim = dims,
+      value = new_value
+    )
+  }
+
+
+  # otherwise just reorder them
 }
 
 # head handles matrices differently to arrays, so explicitly handle 2D greta
 # arrays
 #' @export
 #' @importFrom utils head
-head.greta_array <- function(x, n = 6L, ...) {
+#' @importFrom utils head.matrix
+head.greta_array <- function(x, n = 6L, ...) { # nolint
 
   stopifnot(length(n) == 1L)
 
-  # if x is matrix-like, take the top n rows
-  if (length(dim(x)) == 2) {
+  # use default behaviour for R < 4.0.0
+  if (getRversion() < "4.0.0") {
 
-    nrx <- nrow(x)
-    if (n < 0L)
-      n <- max(nrx + n, 0L)
-    else
-      n <- min(n, nrx)
+    # if x is matrix-like, take the top n rows
+    if (length(dim(x)) == 2) {
+      nrx <- nrow(x)
+      if (n < 0L) {
+        n <- max(nrx + n, 0L)
+      } else {
+        n <- min(n, nrx)
+      }
 
-    ans <- x[seq_len(n), , drop = FALSE]
+      ans <- x[seq_len(n), , drop = FALSE]
+    } else {
+      # otherwise, take the first n elements
 
-  } else {
-    # otherwise, take the first n elements
+      if (n < 0L) {
+        n <- max(length(x) + n, 0L)
+      } else {
+        n <- min(n, length(x))
+      }
 
-    if (n < 0L)
-      n <- max(length(x) + n, 0L)
-    else
-      n <- min(n, length(x))
+      ans <- x[seq_len(n)]
+    }
 
-    ans <- x[seq_len(n)]
+  } else if (getRversion() >= "4.0.0") {
+
+    ans <- head.matrix(x, n, ...)
 
   }
-
-  ans
-
+    ans
 }
 
 #' @export
 #' @importFrom utils tail
-tail.greta_array <- function(x, n = 6L, ...) {
+#' @importFrom utils tail.matrix
+tail.greta_array <- function(x, n = 6L, ...) { # nolint
 
   stopifnot(length(n) == 1L)
 
-  # if x is matrix-like, take the top n rows
-  if (length(dim(x)) == 2) {
+  # use default behaviour for R < 4.0.0
+  if (getRversion() < "4.0.0") {
 
-    nrx <- nrow(x)
+    # if x is matrix-like, take the top n rows
+    if (length(dim(x)) == 2) {
+      nrx <- nrow(x)
 
-    if (n < 0L)
-      n <- max(nrx + n, 0L)
-    else
-      n <- min(n, nrx)
+      if (n < 0L) {
+        n <- max(nrx + n, 0L)
+      } else {
+        n <- min(n, nrx)
+      }
 
-    sel <- as.integer(seq.int(to = nrx, length.out = n))
-    ans <- x[sel, , drop = FALSE]
+      sel <- as.integer(seq.int(to = nrx, length.out = n))
+      ans <- x[sel, , drop = FALSE]
+    } else {
+      # otherwise, take the first n elements
 
-  } else {
-    # otherwise, take the first n elements
+      xlen <- length(x)
 
-    xlen <- length(x)
+      if (n < 0L) {
+        n <- max(xlen + n, 0L)
+      } else {
+        n <- min(n, xlen)
+      }
 
-    if (n < 0L)
-      n <- max(xlen + n, 0L)
-    else
-      n <- min(n, xlen)
+      ans <- x[seq.int(to = xlen, length.out = n)]
+    }
 
-    ans <- x[seq.int(to = xlen, length.out = n)]
+  } else if (getRversion() >= "4.0.0") {
+
+    ans <- tail.matrix(x, n, ...)
 
   }
-
-  ans
-
+    ans
 }
 
 #' @rdname overloaded
 #' @export
-diag <- function(x = 1, nrow, ncol)
+diag <- function(x = 1, nrow, ncol) {
   UseMethod("diag", x)
+}
 
 # wrapper function to avoid a CRAN check warning about using a .Internal() call
 #' @export
-diag.default <- function(...)
+diag.default <- function(...) {
   base::diag(...)
+}
 
 #' @export
 diag.greta_array <- function(x = 1, nrow, ncol) {
-
   dim <- dim(x)
 
   # check the rank isn't too high
   if (length(dim) != 2) {
-    stop("cannot only extract the diagonal from a node ",
-         "with exactly two dimensions")
+    msg <- cli::format_error(
+      "cannot only extract the diagonal from a node with exactly two \\
+      dimensions"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
   }
 
   if (dim[1] != dim[2]) {
-    stop("diagonal elements can only be extracted from square matrices")
+    msg <- cli::format_error(
+      "diagonal elements can only be extracted from square matrices"
+    )
+    stop(
+      msg,
+      call. = FALSE
+    )
   }
 
   # return the dimensions
@@ -629,7 +789,7 @@ diag.greta_array <- function(x = 1, nrow, ncol) {
 
   # return the extraction op
   op("diag", x,
-     dim = dims,
-     tf_operation = "tf$linalg$diag_part")
-
+    dim = dims,
+    tf_operation = "tf$linalg$diag_part"
+  )
 }
