@@ -29,28 +29,33 @@
 #' This help file lists the available internals, but they are not fully
 #' documented and are subject to change and deprecation without warning (though
 #' care will be taken not to break dependent packages on CRAN). For an overview
-#' of how greta works internally, see the \emph{technical details} vignette. See
-#' \url{https://github.com/greta-dev} for examples of R packages extending and
+#' of how greta works internally, see the *technical details* vignette. See
+#' <https://github.com/greta-dev> for examples of R packages extending and
 #' building on greta.
 #'
 #' Please get in contact via GitHub if you want to develop an extension to
 #' greta and need more details of how to use these internal functions.
 #'
-#' You can use \code{attach()} to put a sublist in the search path. E.g.
-#' \code{attach(.internals$nodes$constructors)} will enable you to call
-#' \code{op()}, \code{vble()} and \code{distrib()} directly.
+#' You can use `attach()` to put a sublist in the search path. E.g.
+#' `attach(.internals$nodes$constructors)` will enable you to call
+#' `op()`, `vble()` and `distrib()` directly.
 NULL
 
-nodes_module <- module(constructors = node_constructors_module,
-                       node_classes = node_classes_module,
-                       distribution_classes = distribution_classes_module,
-                       mixture_classes = mixture_module,
-                       joint_classes = joint_module)
+nodes_module <- module(
+  constructors = node_constructors_module,
+  node_classes = node_classes_module,
+  distribution_classes = distribution_classes_module,
+  mixture_classes = mixture_module,
+  joint_classes = joint_module
+)
 
 #' @export
-.internals <- module(greta_arrays = greta_array_module,
-                     nodes = nodes_module,
-                     inference = inference_module,
-                     tensors = tf_functions_module,
-                     utils = utilities_module,
-                     greta_stash)
+.internals <- module(
+  greta_arrays = greta_array_module,
+  nodes = nodes_module,
+  inference = inference_module,
+  tensors = tf_functions_module,
+  utils = utilities_module,
+  checks = checks_module,
+  greta_stash
+)
